@@ -559,12 +559,12 @@ _xdg_mime_magic_matchlet_compare_level (XdgMimeMagicMatchlet *matchlet,
 					size_t                len,
 					int                   indent)
 {
-  while (matchlet != NULL && matchlet->indent == indent)
+  while ((matchlet != NULL) && (matchlet->indent == indent))
     {
       if (_xdg_mime_magic_matchlet_compare_to_data (matchlet, data, len))
 	{
-	  if (matchlet->next == NULL ||
-	      matchlet->next->indent <= indent)
+	  if ((matchlet->next == NULL) ||
+	      (matchlet->next->indent <= indent))
 	    return TRUE;
 
 	  if (_xdg_mime_magic_matchlet_compare_level (matchlet->next,
@@ -604,7 +604,7 @@ _xdg_mime_magic_insert_match (XdgMimeMagic      *mime_magic,
       return;
     }
 
-  if (match->priority < mime_magic->match_list->priority)
+  if (match->priority > mime_magic->match_list->priority)
     {
       match->next = mime_magic->match_list;
       mime_magic->match_list = match;
@@ -614,7 +614,7 @@ _xdg_mime_magic_insert_match (XdgMimeMagic      *mime_magic,
   list = mime_magic->match_list;
   while (list->next != NULL)
     {
-      if (list->next->priority > match->priority)
+      if (list->next->priority < match->priority)
 	{
 	  match->next = list->next;
 	  list->next = match;
@@ -640,7 +640,7 @@ _xdg_mime_magic_free (XdgMimeMagic *mime_magic)
 }
 
 int
-_xdg_mime_magic_get_max_buffer_extents (XdgMimeMagic *mime_magic)
+_xdg_mime_magic_get_buffer_extents (XdgMimeMagic *mime_magic)
 {
   return mime_magic->max_extent;
 }
@@ -686,6 +686,32 @@ _xdg_mime_update_mime_magic_extents (XdgMimeMagic *mime_magic)
   mime_magic->max_extent = max_extent;
 }
 
+static XdgMimeMagicMatchlet *
+_xdg_mime_magic_matchlet_mirror (XdgMimeMagicMatchlet *matchlets)
+{
+  XdgMimeMagicMatchlet *new_list;
+  XdgMimeMagicMatchlet *tmp;
+  
+  if ((matchlets == NULL) || (matchlets->next == NULL)) 
+    {
+      return matchlets;
+    }
+
+  new_list = NULL;
+  tmp = matchlets;
+  while (tmp != NULL) 
+    {
+      XdgMimeMagicMatchlet *matchlet;
+      matchlet = tmp;
+      tmp = tmp->next;
+      matchlet->next = new_list;
+      new_list = matchlet;
+    }
+
+  return new_list;
+  
+}
+
 static void
 _xdg_mime_magic_read_magic_file (XdgMimeMagic *mime_magic,
 				 FILE         *magic_file)
@@ -707,8 +733,11 @@ _xdg_mime_magic_read_magic_file (XdgMimeMagic *mime_magic,
 	  break;
 	case XDG_MIME_MAGIC_MAGIC:
 	  state = _xdg_mime_magic_parse_magic_line (magic_file, match);
-	  if (state == XDG_MIME_MAGIC_SECTION)
-	    _xdg_mime_magic_insert_match (mime_magic, match);
+	  if (state == XDG_MIME_MAGIC_SECTION) 
+	    {
+	      match->matchlet = _xdg_mime_magic_matchlet_mirror (match->matchlet);
+	      _xdg_mime_magic_insert_match (mime_magic, match);
+	    }
 	  else if (state == XDG_MIME_MAGIC_EOF || state == XDG_MIME_MAGIC_ERROR)
 	    _xdg_mime_magic_match_free (match);
 	  break;
